@@ -50,6 +50,9 @@ pub enum RangeMode {
 /// dr.add(35, 90).unwrap();
 /// dr.add(91, 110).unwrap();
 /// dr.add(115, 120).unwrap();
+///
+/// assert!(dr.includes(35));
+/// assert!(!dr.includes(113));
 /// ```
 pub struct DisjointRange {
     ranges: Vec<Range>,
@@ -93,6 +96,18 @@ impl DisjointRange {
         }
     }
 
+    /// Checks whether the given value falls into one of the ranges.
+    pub fn includes(&self, value: usize) -> bool {
+        self.lookup(value).is_some()
+    }
+
+    /// Checks if the data structure contains the provided value.
+    ///
+    /// If yes, return the range that contains this value.
+    pub fn lookup(&self, value: usize) -> Option<&Range> {
+        self.ranges.iter().find(|r| r.includes(value))
+    }
+
     /// Iterates this [`DisjointRange`] in range-ascending order.
     pub fn iter(self: &Self) -> DisjointRangeIter {
         DisjointRangeIter::new(self)
@@ -125,14 +140,167 @@ impl<'a> Iterator for DisjointRangeIter<'a> {
     }
 }
 
+/// An enumeration of possible range comparison outputs.
+///
+/// This is used in conjunction with [`Range::compare()`].
+///
+/// Depending on their [`RangeMode`], two range's comparison results could
+/// differ, even though the ranges have the same min and max values.
+///
+/// ```rust
+/// use drng::{Range, RangeMode, RangeCompareResult::*};
+///
+/// let r1 = Range::new(100, 200, RangeMode::Inclusive).unwrap();
+/// let r2 = Range::new(200, 300, RangeMode::Inclusive).unwrap();
+/// let r2_start_excl = Range::new(200, 300, RangeMode::StartExclusive).unwrap();
+///
+/// // r1 overlaps with the lower part of r2
+/// assert_eq!(Range::compare(&r1, &r2), OverlapLower);
+///
+/// // r1 is strictly lower than r2_start_excl
+/// assert_eq!(Range::compare(&r1, &r2_start_excl), LessThanNoOverlap);
+/// ```
 #[derive(Debug, PartialEq, Eq)]
-enum RangeCompareResult {
+pub enum RangeCompareResult {
+    /// The first range is strictly lower than the second range. They do not
+    /// overlap with one another.
+    ///
+    /// ```txt
+    /// First:    xxxxxxx
+    /// Second:              xxxxxx
+    /// ```
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let first  = Range::new(10, 20, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(200, 300, RangeMode::Inclusive).unwrap();
+    ///
+    /// assert_eq!(Range::compare(&first, &second), LessThanNoOverlap);
+    /// ```
     LessThanNoOverlap,
-    OverlapUpper,
-    Contained,
-    Contains,
-    Equal,
+    /// The first range overlaps with the lower part of the second range.
+    ///
+    /// ```txt
+    /// First:  xxxxxxxx
+    /// Second:     xxxxxxxx
+    /// ```
+    ///
+    /// Note that once the first range's end-exclusive value equals the second
+    /// range's end-exclusive value, the first range no longer overlaps with
+    /// the second. In this case, the first one "contains" the second.
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let second = Range::new(200, 300, RangeMode::Inclusive).unwrap();
+    ///
+    /// let first  = Range::new(100, 299, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), OverlapLower);
+    ///
+    /// let first  = Range::new(100, 300, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), Contains);
+    /// ```
     OverlapLower,
+    /// The first range is strictly smaller than the second range.
+    ///
+    /// ```txt
+    /// First:         xxx
+    /// Second:    xxxxxxxxxxxxx
+    /// ```
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let first = Range::new(33, 35, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(10, 990, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), Contained);
+    /// assert_eq!(Range::compare(&second, &first), Contains);
+    ///
+    /// // the first and second range share the same starting point
+    /// let first = Range::new(10, 35, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(10, 990, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), Contained);
+    /// assert_eq!(Range::compare(&second, &first), Contains);
+    /// ```
+    Contained,
+    /// The second range is strictly larger than the first range.
+    ///
+    /// ```txt
+    /// First:    xxxxxxxxxxxxx
+    /// Second:         xxx
+    /// ```
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let first = Range::new(10, 990, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(33, 35, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), Contains);
+    /// assert_eq!(Range::compare(&second, &first), Contained);
+    ///
+    /// // the first and second range share the same starting point
+    /// let first = Range::new(10, 990, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(10, 35, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), Contains);
+    /// assert_eq!(Range::compare(&second, &first), Contained);
+    /// ```
+    Contains,
+    /// The first range and the second range are range-equal.
+    ///
+    /// ```txt
+    /// First:  xxxxxxx
+    /// Second: xxxxxxx
+    /// ```
+    ///
+    /// Depending on their [`RangeMode`], two ranges could equal one another
+    /// even though their min and max value are not exactly the same.
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let first = Range::new(10, 15, RangeMode::Inclusive).unwrap();
+    /// let first_end_excl = Range::new(10, 16, RangeMode::EndExclusive).unwrap();
+    /// let first_start_excl = Range::new(9, 15, RangeMode::StartExclusive).unwrap();
+    /// let first_excl = Range::new(9, 16, RangeMode::Exclusive).unwrap();
+    /// let second = Range::new(10, 15, RangeMode::Inclusive).unwrap();
+    ///
+    /// assert_eq!(Range::compare(&first, &second), Equal);
+    /// assert_eq!(Range::compare(&first_end_excl, &second), Equal);
+    /// assert_eq!(Range::compare(&first_start_excl, &second), Equal);
+    /// assert_eq!(Range::compare(&first_excl, &second), Equal);
+    /// ```
+    Equal,
+    /// The first range overlaps with the upper part of the second range.
+    ///
+    /// ```txt
+    /// First:     xxxxxxx
+    /// Second:  xxxxx
+    /// ```
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult};
+    ///
+    /// let first = Range::new(13, 22, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(10, 15, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), RangeCompareResult::OverlapUpper);
+    /// ```
+    OverlapUpper,
+    /// The first range is strictly greater than the second range. They do
+    /// not overlap one another.
+    ///
+    /// ```txt
+    /// First:          xxxxxxx
+    /// Second: xxxxx
+    /// ```
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let first = Range::new(20, 25, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(10, 15, RangeMode::Inclusive).unwrap();
+    /// assert_eq!(Range::compare(&first, &second), GreaterNoOverlap);
+    /// ```
     GreaterNoOverlap,
 }
 
@@ -152,6 +320,24 @@ pub struct Range {
     max_incl: usize,
 }
 
+impl Range {
+    /// Compares two ranges.
+    ///
+    /// ```
+    /// use drng::{Range, RangeMode, RangeCompareResult::*};
+    ///
+    /// let first  = Range::new(10, 20, RangeMode::Inclusive).unwrap();
+    /// let second = Range::new(15, 33, RangeMode::Inclusive).unwrap();
+    ///
+    /// assert_eq!(Range::compare(&first, &second), OverlapLower);
+    /// ```
+    ///
+    /// For more documentation, see [`RangeCompareResult`].
+    pub fn compare(r1: &Range, r2: &Range) -> RangeCompareResult {
+        r1.compare_with(r2)
+    }
+}
+
 impl Range {
     /// Constructs a new [`Range`].
     pub fn new(min: usize, max: usize, mode: RangeMode) -> Result<Self, BadRange> {
@@ -194,6 +380,10 @@ impl Range {
         self.max_incl
     }
 
+    fn includes(&self, value: usize) -> bool {
+        value >= self.min_incl && value <= self.max_incl
+    }
+
     fn compare_with(&self, other: &Range) -> RangeCompareResult {
         let other_min_incl = other.min_incl();
         let other_max_incl = other.max_incl();
@@ -377,6 +567,36 @@ mod tests {
         );
     }
 
+    #[test]
+    fn find_value_in_range() {
+        let mut dr = DisjointRange::new(RangeMode::EndExclusive);
+
+        dr.add(100, 200).unwrap();
+        assert!(dr.includes(100));
+        assert!(dr.includes(105));
+        assert!(dr.includes(199));
+        assert!(!dr.includes(200));
+        assert!(!dr.includes(201));
+        assert!(!dr.includes(3));
+        assert!(!dr.includes(30000));
+    }
+
+    #[test]
+    fn lookup_range() {
+        let mut dr = DisjointRange::new(RangeMode::EndExclusive);
+
+        insert_ranges(&mut dr, &[[100, 200], [300, 4500]]);
+
+        let found_fst = dr.lookup(199).unwrap();
+        assert_range_min_max(found_fst, 100, 200);
+
+        assert!(dr.lookup(200).is_none());
+        assert!(dr.lookup(248).is_none());
+
+        let found_snd = dr.lookup(344).unwrap();
+        assert_range_min_max(found_snd, 300, 4500);
+    }
+
     fn insert_ranges(dr: &mut DisjointRange, seq: &[[usize; 2]]) {
         seq.iter().for_each(|[min, max]| {
             dr.add(*min, *max)
